@@ -1,7 +1,7 @@
 # coding=utf-8
 #
 # catkin_lint
-# Copyright (c) 2013-2018 Fraunhofer FKIE
+# Copyright (c) 2013-2020 Fraunhofer FKIE
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -30,11 +30,11 @@
 
 import os
 import posixpath
-import re
 import stat
-from ..linter import ERROR, WARNING, NOTICE, PathConstants
+from ..linter import ERROR, WARNING, NOTICE, PathConstants, PathClass
 from ..cmake import argparse as cmake_argparse
-from ..util import iteritems, is_sorted
+from ..util import iteritems, is_sorted, enumerate_package_files
+from ..environment import PackageType
 from .manifest import depends as manifest_depends
 from functools import partial
 
@@ -53,8 +53,21 @@ def includes(linter):
         includes = set([info.source_relative_path(d) for d in args])
         info.build_includes |= includes
 
+    def on_target_include_directories(info, cmd, args):
+        opts, args = cmake_argparse(args, {"BEFORE": "-", "SYSTEM": "-", "PRIVATE": "*", "PUBLIC": "*", "INTERFACE": "*"})
+        for scope in ["PUBLIC", "PRIVATE", "INTERFACE"]:
+            for incl in opts[scope]:
+                if not info.is_valid_path(incl):
+                    info.report(WARNING, "EXTERNAL_DIRECTORY", cmd=cmd, directory=info.report_path(incl))
+                elif scope != "PRIVATE" and not info.is_valid_path(incl, valid=[PathClass.SOURCE]):
+                    info.report(WARNING, "EXTERNAL_INTERFACE_PATH", cmd=cmd, scope=scope, directory=info.report_path(incl))
+                if not info.is_existing_path(incl, check=os.path.isdir):
+                    info.report(ERROR, "MISSING_DIRECTORY", cmd=cmd, directory=info.report_path(incl))
+                info.build_includes.add(info.source_relative_path(incl))
+
     linter.add_init_hook(on_init)
     linter.add_command_hook("include_directories", on_include_directories)
+    linter.add_command_hook("target_include_directories", on_target_include_directories)
 
 
 def targets(linter):
@@ -85,8 +98,10 @@ def targets(linter):
             info.target_links[args[0]] = set()
 
     def on_target_link_libraries(info, cmd, args):
+        opts, args = cmake_argparse(args, {"PUBLIC": "*", "PRIVATE": "*", "INTERFACE": "*"})
+        deps = opts["PUBLIC"] + opts["PRIVATE"] + opts["INTERFACE"] or args[1:]
         if args[0] in info.target_links:
-            info.target_links[args[0]] |= set([d for d in args[1:] if not os.path.isabs(d)])
+            info.target_links[args[0]] |= set([d for d in deps if not os.path.isabs(d)])
         else:
             info.target_order_violation.add(args[0])
 
@@ -193,13 +208,24 @@ def source_files(linter):
 
 def link_directories(linter):
     def on_link_directories(info, cmd, args):
+        _, args = cmake_argparse(args, {"BEFORE": "-", "AFTER": "-"})
         externals = [p for p in args if not info.is_internal_path(p)]
         if externals:
             info.report(ERROR, "EXTERNAL_LINK_DIRECTORY")
         else:
             info.report(WARNING, "LINK_DIRECTORY")
 
+    def on_target_link_directories(info, cmd, args):
+        opts, args = cmake_argparse(args, {"BEFORE": "-", "PUBLIC": "*", "PRIVATE": "*", "INTERFACE": "*"})
+        externals = opts["PUBLIC"] + opts["PRIVATE"] + opts["INTERFACE"] or args[1:]
+        externals = [p for p in externals if not info.is_internal_path(p)]
+        if externals:
+            info.report(ERROR, "EXTERNAL_LINK_DIRECTORY")
+        else:
+            info.report(WARNING, "LINK_DIRECTORY")
+
     linter.add_command_hook("link_directories", on_link_directories)
+    linter.add_command_hook("target_link_directories", on_target_link_directories)
 
 
 def depends(linter):
@@ -211,7 +237,13 @@ def depends(linter):
         info.checked_packages = set()
 
     def on_find_package(info, cmd, args):
-        opts, args = cmake_argparse(args, {"REQUIRED": "-", "COMPONENTS": "*", "OPTIONAL_COMPONENTS": "*"})
+        opts, args = cmake_argparse(args, {
+            "EXACT": "-", "QUIET": "-", "MODULE": "-", "NO_POLICY_SCOPE": "-", "REQUIRED": "-", "COMPONENTS": "*", "OPTIONAL_COMPONENTS": "*",
+            "NAMES": "*", "CONFIGS": "*", "HINTS": "*", "PATHS": "*", "PATH_SUFFIXES": "*", "NO_DEFAULT_PATH": "-", "NO_PACKAGE_ROOT_PATH": "-",
+            "NO_CMAKE_PATH": "-", "NO_CMAKE_ENVIRONMENT_PATH": "-", "NO_SYSTEM_ENVIRONMENT_PATH": "-", "NO_CMAKE_PACKAGE_REGISTRY": "-",
+            "NO_CMAKE_BUILDS_PATH": "-", "NO_CMAKE_SYSTEM_PATH": "-", "NO_CMAKE_SYSTEM_PACKAGE_REGISTRY": "-", "CMAKE_FIND_ROOT_PATH_BOTH": "-",
+            "ONLY_CMAKE_FIND_ROOT_PATH": "-", "NO_CMAKE_FIND_ROOT_PATH": "-"
+        })
         this_components = opts["COMPONENTS"] if opts["COMPONENTS"] else args[1:]
         if "project" not in info.commands:
             info.report(ERROR, "ORDER_VIOLATION", first_cmd=cmd, second_cmd="project")
@@ -257,14 +289,13 @@ def depends(linter):
                     info.report(ERROR, "SHADOWED_FIND", pkg=pkg)
                 else:
                     info.report(WARNING, "DUPLICATE_FIND", pkg=pkg)
-            if not info.env.is_known_pkg(pkg):
-                if info.env.ok:
-                    info.report(ERROR, "UNKNOWN_PACKAGE", pkg=pkg)
-            elif not info.env.is_catkin_pkg(pkg):
-                if info.env.ok:
-                    info.report(ERROR, "NO_CATKIN_COMPONENT", pkg=pkg)
+            pkg_type = info.env.get_package_type(pkg)
+            if pkg_type == PackageType.UNKNOWN:
+                info.report(ERROR, "UNKNOWN_PACKAGE", pkg=pkg)
+            elif pkg_type == PackageType.OTHER:
+                info.report(ERROR, "NO_CATKIN_COMPONENT", pkg=pkg)
         for pkg in args[1:]:
-            if info.env.is_known_pkg(pkg):
+            if info.env.get_package_type(pkg) & PackageType.ANY:
                 info.report(NOTICE, "MISSING_COMPONENTS", pkg=pkg)
         info.find_packages |= set(this_components)
         info.required_packages |= set(this_components)
@@ -277,13 +308,13 @@ def depends(linter):
 
     def on_final(info):
         for pkg in info.required_packages - info.build_dep - info.buildtool_dep - info.test_dep:
-            if info.env.is_known_pkg(pkg):
+            if info.env.get_package_type(pkg) & PackageType.ANY and pkg != "catkin":
                 info.report(ERROR, "MISSING_DEPEND", pkg=pkg, type="build", file_location=("package.xml", 0))
         for pkg in info.find_packages - info.required_packages - info.checked_packages:
-            if info.env.is_catkin_pkg(pkg):
+            if info.env.get_package_type(pkg) == PackageType.CATKIN:
                 info.report(ERROR, "MISSING_REQUIRED", pkg=pkg, file_location=("CMakeLists.txt", 0))
         for pkg in info.build_dep - (info.find_packages - info.test_packages):
-            if info.env.is_catkin_pkg(pkg):
+            if info.env.get_package_type(pkg) == PackageType.CATKIN:
                 # Ignore pure Python packages (or at least packages that look like it), following the
                 # Jack O'Quin heuristic from issue fkie/catkin_lint#22
                 if pkg not in info.pkg_modules and (info.executables or info.libraries or "catkin_python_setup" not in info.commands):
@@ -323,21 +354,23 @@ def exports(linter):
         info.export_targets = set()
 
     def on_catkin_package(info, cmd, args):
-        opts, args = cmake_argparse(args, {"INCLUDE_DIRS": "*", "LIBRARIES": "*", "DEPENDS": "*", "CATKIN_DEPENDS": "*", "CFG_EXTRAS": "*", "EXPORTED_TARGETS": "*"})
+        opts, args = cmake_argparse(args, {
+            "INCLUDE_DIRS": "*", "LIBRARIES": "*", "DEPENDS": "*", "CATKIN_DEPENDS": "*", "CFG_EXTRAS": "*", "EXPORTED_TARGETS": "*",
+            "SKIP_CMAKE_CONFIG_GENERATION": "-", "SKIP_PKG_CONFIG_GENERATION": "-"
+        })
         for list_name in ["CATKIN_DEPENDS", "DEPENDS", "CFG_EXTRAS", "EXPORTED_TARGETS"]:
             if not is_sorted(opts[list_name]):
                 info.report(NOTICE, "UNSORTED_LIST", name=list_name)
         for pkg in opts["CATKIN_DEPENDS"]:
-            if not info.env.is_known_pkg(pkg):
-                if info.env.ok:
-                    info.report(ERROR, "UNKNOWN_PACKAGE", pkg=pkg)
-            elif not info.env.is_catkin_pkg(pkg):
-                if info.env.ok:
-                    info.report(ERROR, "SYSTEM_AS_CATKIN_DEPEND", pkg=pkg)
+            pkg_type = info.env.get_package_type(pkg)
+            if pkg_type == PackageType.UNKNOWN:
+                info.report(ERROR, "UNKNOWN_PACKAGE", pkg=pkg)
+            elif pkg_type == PackageType.OTHER:
+                info.report(ERROR, "SYSTEM_AS_CATKIN_DEPEND", pkg=pkg)
         for pkg in opts["DEPENDS"]:
-            if info.env.is_catkin_pkg(pkg):
-                if info.env.ok:
-                    info.report(ERROR, "CATKIN_AS_SYSTEM_DEPEND", pkg=pkg)
+            pkg_type = info.env.get_package_type(pkg)
+            if pkg_type == PackageType.CATKIN:
+                info.report(ERROR, "CATKIN_AS_SYSTEM_DEPEND", pkg=pkg)
             elif pkg in info.pkg_modules_prefix:
                 info.report(ERROR, "EXPORTED_PKG_CONFIG", pkg=pkg)
             elif pkg not in info.find_packages and not ("%s_INCLUDE_DIRS" % pkg in info.var and "%s_LIBRARIES" % pkg in info.var):
@@ -353,15 +386,20 @@ def exports(linter):
 
     def on_final(info):
         for pkg in info.export_packages - info.export_dep:
-            if info.env.is_known_pkg(pkg):
+            if info.env.get_package_type(pkg) & PackageType.ANY:
                 if pkg == "message_runtime":
                     if pkg not in info.exec_dep:
                         info.report(ERROR, "MISSING_DEPEND", pkg=pkg, type="run" if info.manifest.package_format < 2 else "exec", file_location=("package.xml", 0))
                 else:
                     info.report(ERROR, "MISSING_DEPEND", pkg=pkg, type="run" if info.manifest.package_format < 2 else "build_export", file_location=("package.xml", 0))
         for pkg in (info.find_packages & info.build_dep & info.export_dep) - info.export_packages:
-            if re.search(r"_(msg|message)s?(_|$)", pkg) and info.env.is_catkin_pkg(pkg):
-                info.report(WARNING, "SUGGEST_CATKIN_DEPEND", pkg=pkg, file_location=info.location_of("catkin_package"))
+            if info.env.get_package_type(pkg) == PackageType.CATKIN:
+                pkg_manifest = info.env.get_manifest(pkg)
+                if pkg_manifest is not None:
+                    for dep in pkg_manifest.exec_depends:
+                        if dep.name == "message_runtime":
+                            info.report(ERROR, "MISSING_CATKIN_DEPEND", pkg=pkg, file_location=info.location_of("catkin_package"))
+                            break
         if info.export_includes and info.libraries and not info.export_libs:
             info.report(WARNING, "MISSING_EXPORT_LIB", file_location=info.location_of("catkin_package"))
         if info.executables or info.libraries:
@@ -411,13 +449,13 @@ def pkg_config(linter):
         info.pkg_modules_prefix = set()
 
     def on_pkg_check_modules(info, cmd, args):
-        opts, args = cmake_argparse(args, {"REQUIRED": "-", "QUIET": "-", "NO_CMAKE_PATH": "-", "NO_CMAKE_ENVIRONMENT_PATH": "-", "IMPORTED_TARGET": "-"})
+        opts, args = cmake_argparse(args, {"REQUIRED": "-", "QUIET": "-", "NO_CMAKE_PATH": "-", "NO_CMAKE_ENVIRONMENT_PATH": "-", "IMPORTED_TARGET": "-", "GLOBAL": "-"})
         info.pkg_modules_prefix.add(args[0])
         for pkg in args[1:]:
             if "=" in pkg:
                 pkg = pkg.split("=")[0].rstrip("<>=")
             info.pkg_modules.add(pkg)
-            if info.env.is_catkin_pkg(pkg):
+            if info.env.get_package_type(pkg) == PackageType.CATKIN:
                 info.report(ERROR, "MISCONFIGURED_CATKIN_PACKAGE", pkg=pkg)
 
     linter.add_init_hook(on_init)
@@ -428,11 +466,11 @@ def installs(linter):
     def on_init(info):
         info.install_targets = set()
         info.install_programs = set()
-        info.install_includes = False
         info.install_files = set()
+        info.has_includes_installed = False
 
     def on_catkin_install_python(info, cmd, args):
-        opts, args = cmake_argparse(args, {"PROGRAMS": "+", "DESTINATION": "!"})
+        opts, args = cmake_argparse(args, {"PROGRAMS": "+", "DESTINATION": "!", "OPTIONAL": "-"})
         for f in opts["PROGRAMS"]:
             if f:
                 if not info.is_valid_path(f):
@@ -444,7 +482,7 @@ def installs(linter):
                             shebang = fd.readline()
                             if not shebang.startswith("#!") or "python" not in shebang:
                                 info.report(ERROR, "MISSING_SHEBANG", file=info.report_path(f), interpreter="python")
-                else:
+                elif not opts["OPTIONAL"]:
                     info.report(ERROR, "MISSING_FILE", cmd=cmd, file=info.report_path(f))
                 info.install_programs.add(info.source_relative_path(f))
         if not info.is_catkin_bin_install_destination(opts["DESTINATION"]):
@@ -452,7 +490,16 @@ def installs(linter):
 
     def on_install(info, cmd, args):
         install_type = None
-        opts, args = cmake_argparse(args, {"PROGRAMS": "*", "FILES": "*", "TARGETS": "*", "DIRECTORY": "*", "DESTINATION": "?", "ARCHIVE DESTINATION": "?", "LIBRARY DESTINATION": "?", "RUNTIME DESTINATION": "?", "USE_SOURCE_PERMISSIONS": "-"})
+        opts, args = cmake_argparse(args, {
+            "EXPORT": "?", "PROGRAMS": "*", "FILES": "*", "TARGETS": "*", "DIRECTORY": "*",
+            "DESTINATION": "?", "ARCHIVE DESTINATION": "?", "LIBRARY DESTINATION": "?", "RUNTIME DESTINATION": "?",
+            "OBJECTS DESTINATION": "?", "FRAMEWORK DESTINATION": "?", "BUNDLE DESTINATION": "?",
+            "PRIVATE_HEADER DESTINATION": "?", "PUBLIC_HEADER DESTINATION": "?", "RESOURCE DESTINATION": "?",
+            "INCLUDES DESTINATION": "*",
+            "PATTERN": "?", "EXCLUDE": "-", "USE_SOURCE_PERMISSIONS": "-", "REGEX": "-",
+            "PERMISSIONS": "*", "CONFIGURATIONS": "*", "COMPONENT": "?", "NAMELINK_COMPONENT": "?", "OPTIONAL": "-",
+            "EXCLUDE_FROM_ALL": "-", "NAMELINK_ONLY": "-", "NAMELINK_SKIP": "-"
+        })
         if opts["PROGRAMS"]:
             install_type = "PROGRAMS"
             for f in opts["PROGRAMS"]:
@@ -470,15 +517,17 @@ def installs(linter):
                         info.report(WARNING, "EXTERNAL_DIRECTORY", cmd=cmd, directory=info.report_path(d))
                     if info.is_existing_path(d, check=os.path.isdir):
                         real_d = info.real_path(info.source_relative_path(d))
+                        _, dir_component = os.path.split(d)
                         if os.path.isdir(real_d):
-                            if opts["USE_SOURCE_PERMISSIONS"]:
-                                for dirpath, _, filenames in os.walk(real_d, topdown=True):
-                                    for filename in filenames:
-                                        real_filename = os.path.join(dirpath, filename)
-                                        pkg_filename = info.source_relative_path(real_filename[len(info.path) + 1:])
-                                        mode = os.stat(real_filename).st_mode
-                                        if mode & stat.S_IXUSR:
-                                            info.install_programs.add(pkg_filename)
+                            for dirpath, filename in enumerate_package_files(real_d, catkin_ignore=False, ignore_unimportant=False):
+                                real_filename = os.path.join(dirpath, filename)
+                                if opts["USE_SOURCE_PERMISSIONS"]:
+                                    pkg_filename = info.source_relative_path(real_filename[len(info.path) + 1:])
+                                    mode = os.stat(real_filename).st_mode
+                                    if mode & stat.S_IXUSR:
+                                        info.install_programs.add(pkg_filename)
+                                relative_filename = os.path.relpath(real_filename, real_d)
+                                info.install_files.add(posixpath.normpath(posixpath.join(PathConstants.CATKIN_INSTALL, opts["DESTINATION"], dir_component, relative_filename)))
                     else:
                         info.report(ERROR, "MISSING_DIRECTORY", cmd=cmd, directory=info.report_path(d))
         if opts["FILES"]:
@@ -492,6 +541,9 @@ def installs(linter):
             info.install_files |= set([posixpath.normpath(posixpath.join(PathConstants.CATKIN_INSTALL, opts["DESTINATION"], posixpath.basename(f))) for f in opts["FILES"]])
         if opts["TARGETS"]:
             install_type = "TARGETS"
+            for target in opts["TARGETS"]:
+                if target not in info.targets:
+                    info.report(ERROR, "UNDEFINED_TARGET", target=target)
             info.install_targets |= set(opts["TARGETS"])
         if install_type is None:
             return
@@ -504,7 +556,7 @@ def installs(linter):
                 elif not info.is_catkin_install_destination(opts[dest]):
                     info.report(WARNING, "WRONG_INSTALL_DESTINATION", type=install_type, dest=dest)
                 if info.is_catkin_install_destination(opts[dest], "include"):
-                    info.install_includes = True
+                    info.has_includes_installed = True
 
     def on_final(info):
         for lib in info.export_libs:
@@ -513,16 +565,19 @@ def installs(linter):
         for tgt in info.executables - info.install_targets:
             if "test" not in tgt.lower() and "example" not in tgt.lower():
                 info.report(WARNING, "UNINSTALLED_TARGET", target=tgt, file_location=("CMakeLists.txt", 0))
-        if info.export_includes and not info.install_includes:
+        include_path = posixpath.normpath(posixpath.join(PathConstants.CATKIN_INSTALL, "include")) + posixpath.sep
+        package_include_path = posixpath.join(include_path, info.manifest.name) + posixpath.sep
+        include_files = [f for f in info.install_files if f.startswith(include_path)]
+        if info.export_includes and not info.has_includes_installed:
             info.report(ERROR if "install" in info.commands else WARNING, "UNINSTALLED_INCLUDE_PATH", file_location=info.location_of("catkin_package"))
+        for f in include_files:
+            if not f.startswith(package_include_path):
+                info.report(NOTICE, "HEADER_OUTSIDE_PACKAGE_INCLUDE_PATH", file=info.report_path(f))
         for target, depends in iteritems(info.target_links):
             if target in info.install_targets:
                 for lib in depends:
                     if lib in info.libraries and lib not in info.install_targets and lib not in info.static_libraries and lib not in info.interface_libraries:
                         info.report(ERROR, "UNINSTALLED_DEPEND", export_target=target, target=lib, file_location=info.location_of("catkin_package"))
-        for target in info.install_targets:
-            if target not in info.targets:
-                info.report(ERROR, "UNDEFINED_TARGET", target=target, file_location=("CMakeLists.txt", 0))
 
     linter.require(targets)
     linter.require(exports)
@@ -591,15 +646,12 @@ def scripts(linter):
         return pkg_filename in info.install_programs or pkg_filename in info.dynamic_reconfigure_files
 
     def on_final(info):
-        for dirpath, dirnames, filenames in os.walk(info.path, topdown=True):
-            for filename in filenames:
-                if "test" not in filename.lower() and "example" not in filename.lower():
-                    full_filename = os.path.join(dirpath, filename)
-                    pkg_filename = info.source_relative_path(full_filename[len(info.path) + 1:])
-                    mode = os.stat(full_filename).st_mode
-                    if mode & stat.S_IXUSR and not is_installed(info, pkg_filename):
-                        info.report(WARNING, "UNINSTALLED_SCRIPT", script=pkg_filename)
-            dirnames[:] = [d for d in dirnames if not d.startswith(".") and "test" not in d and "build" not in d and "example" not in d]
+        for dirpath, filename in enumerate_package_files(info.path):
+            full_filename = os.path.join(dirpath, filename)
+            pkg_filename = info.source_relative_path(full_filename[len(info.path) + 1:])
+            mode = os.stat(full_filename).st_mode
+            if mode & stat.S_IXUSR and not is_installed(info, pkg_filename):
+                info.report(WARNING, "UNINSTALLED_SCRIPT", script=pkg_filename)
 
     linter.require(installs)
     linter.require(dynamic_reconfigure)
