@@ -1,7 +1,7 @@
 # coding=utf-8
 #
 # catkin_lint
-# Copyright (c) 2013-2018 Fraunhofer FKIE
+# Copyright (c) 2013-2020 Fraunhofer FKIE
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -94,6 +94,9 @@ def find_packages(basepath, use_cache=True):
         packages[path] = manifest
     if cache_updated:
         _store_cache()
+    for package in packages.values():
+        if hasattr(package, "evaluate_conditions"):
+            package.evaluate_conditions(os.environ)
     return packages
 
 
@@ -106,13 +109,21 @@ def is_catkin_package(manifest):
     return True
 
 
+class PackageType:
+    UNKNOWN = 0
+    CATKIN = 1
+    OTHER = 2
+    ANY = 3  # CATKIN | OTHER
+    INDETERMINATE = 32768
+
+
 class CatkinEnvironment(object):
     def __init__(self, os_env=None, use_rosdep=True, use_rosdistro=True, use_cache=True, quiet=False):
         self.package_path_order = []
         self.searched_paths = {}
         self.known_catkin_pkgs = set()
         self.known_other_pkgs = set()
-        self.ok = True
+        self.knows_everything = use_rosdep and use_rosdistro
         self.os_env = os_env
         self.use_cache = use_cache
         self.use_rosdistro = use_rosdistro
@@ -127,7 +138,7 @@ class CatkinEnvironment(object):
                 if not self.quiet:
                     sys.stderr.write("catkin_lint: cannot load rosdep database: %s\n" % str(err))
                     sys.stderr.write("catkin_lint: unknown dependencies will be ignored\n")
-                self.ok = False
+                self.knows_everything = False
             finally:
                 gc.enable()
 
@@ -157,19 +168,38 @@ class CatkinEnvironment(object):
                     return p, m
         raise KeyError()
 
+    @property
+    def ok(self):
+        if not self.quiet:  # pragma: no cover
+            sys.stderr.write("catkin_lint: deprecated access to env.ok, use env.get_package_type() instead\n")
+        return self.knows_everything
+
     def is_catkin_pkg(self, name):
+        if not self.quiet:  # pragma: no cover
+            sys.stderr.write("catkin_lint: deprecated call to env.is_catkin_pkg(), use env.get_package_type() instead\n")
+        return self.get_package_type(name) == PackageType.CATKIN
+
+    def is_known_pkg(self, name):
+        if not self.quiet:  # pragma: no cover
+            sys.stderr.write("catkin_lint: deprecated call to env.is_known_pkg(), use env.get_package_type() instead\n")
+        return self.get_package_type(name) & PackageType.ANY
+
+    def get_package_type(self, name):
         if name in self.known_catkin_pkgs:
-            return True
+            return PackageType.CATKIN
         if name in self.known_other_pkgs:
-            return False
+            return PackageType.OTHER
         if self.rosdep is not None:
-            if not self.rosdep.is_ros(name):
-                return False
-            try:
-                return is_catkin_package(self.get_manifest(name))
-            except (IOError, KeyError):
-                return True
-        return False
+            if self.rosdep.has_key(name):  # noqa
+                try:
+                    if self.rosdep.is_ros(name):
+                        manifest = self.get_manifest(name)
+                        if manifest is not None and is_catkin_package(manifest):
+                            return PackageType.CATKIN
+                    return PackageType.OTHER
+                except (IOError, KeyError):
+                    return PackageType.INDETERMINATE
+        return PackageType.UNKNOWN if self.knows_everything else PackageType.INDETERMINATE
 
     def get_manifest(self, name):
         global _cache
@@ -198,9 +228,9 @@ class CatkinEnvironment(object):
             if self.rosdistro is None:
                 self.rosdistro = get_rosdistro(quiet=self.quiet)
             if not self.rosdistro.ok():
-                if not self.quiet and self.ok:
+                if not self.quiet and self.knows_everything:
                     sys.stderr.write("catkin_lint: unknown dependencies will be ignored\n")
-                self.ok = False
+                self.knows_everything = False
                 raise KeyError()
             manifest = self.rosdistro.download_manifest(name)
             if self.use_cache:
@@ -209,19 +239,12 @@ class CatkinEnvironment(object):
             return manifest
         raise KeyError()
 
-    def is_known_pkg(self, name):
-        if name in self.known_catkin_pkgs or name in self.known_other_pkgs:
-            return True
-        if self.rosdep is not None:
-            return self.rosdep.has_key(name)  # noqa
-        return False
-
 
 _cache = None
-try:
+try:  # pragma: no cover
     from rospkg import get_ros_home
     _cache_dir = os.path.join(get_ros_home(), "catkin_lint")
-except ImportError:
+except ImportError:  # pragma: no cover
     _cache_dir = os.path.join(os.path.expanduser("~"), ".ros", "catkin_lint")
 
 
@@ -233,7 +256,7 @@ def _load_cache():
             gc.disable()
             with open(os.path.join(_cache_dir, "packages.pickle"), "rb") as f:
                 _cache = pickle.loads(f.read())
-                if not isinstance(_cache, Cache) or _cache.version != 1:
+                if not isinstance(_cache, Cache) or _cache.version != CACHE_VERSION:
                     raise RuntimeError()
         except Exception:
             _cache = Cache()
