@@ -1,3 +1,33 @@
+# coding=utf-8
+#
+# catkin_lint
+# Copyright (c) 2013-2020 Fraunhofer FKIE
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+#  * Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+#  * Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+#  * Neither the name of the Fraunhofer organization nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 import unittest
 import catkin_lint.checks.build as cc
 from .helper import create_env, create_manifest, create_manifest2, mock_lint, patch, mock_open, posix_and_nt
@@ -19,12 +49,20 @@ class ChecksBuildTest(unittest.TestCase):
         self.assertEqual([], result)
         result = mock_lint(env, pkg, "include_directories(/some/hardcoded/path)", checks=cc.includes)
         self.assertEqual(["EXTERNAL_DIRECTORY"], result)
+        result = mock_lint(env, pkg, "target_include_directories(mock PRIVATE /some/hardcoded/path)", checks=cc.includes)
+        self.assertEqual(["EXTERNAL_DIRECTORY"], result)
+        result = mock_lint(env, pkg, "target_include_directories(mock PUBLIC /some/hardcoded/path)", checks=cc.includes)
+        self.assertEqual(["EXTERNAL_DIRECTORY"], result)
+        result = mock_lint(env, pkg, "find_package(catkin REQUIRED) target_include_directories(mock PUBLIC ${catkin_INCLUDE_DIRS})", checks=cc.includes)
+        self.assertEqual(["EXTERNAL_INTERFACE_PATH"], result)
         result = mock_lint(env, pkg, "find_package(catkin REQUIRED) include_directories(${catkin_INCLUDE_DIRS})", checks=cc.includes)
         self.assertEqual([], result)
         result = mock_lint(env, pkg, "include_directories(missing_include)", checks=cc.includes)
         self.assertEqual(["MISSING_DIRECTORY"], result)
         result = mock_lint(env, pkg, "include_directories(/some/hardcoded/but/missing/path)", checks=cc.includes)
         self.assertEqual(["EXTERNAL_DIRECTORY", "MISSING_DIRECTORY"], result)
+        result = mock_lint(env, pkg, "target_include_directories(mock PRIVATE /some/hardcoded/but/missing/path)", checks=cc.includes)
+        self.assertEqual(["EXTERNAL_DIRECTORY", "MISSING_DIRECTORY"], result)
         result = mock_lint(env, pkg, "include_directories(${CATKIN_DEVEL_PREFIX}/include)", checks=cc.includes)
         self.assertEqual(["EXTERNAL_DIRECTORY"], result)
         result = mock_lint(env, pkg, "include_directories(${CATKIN_INSTALL_PREFIX}/include)", checks=cc.includes)
@@ -67,8 +105,12 @@ class ChecksBuildTest(unittest.TestCase):
         pkg = create_manifest("mock")
         result = mock_lint(env, pkg, "link_directories(in_package)", checks=cc.link_directories)
         self.assertEqual(["LINK_DIRECTORY"], result)
+        result = mock_lint(env, pkg, "target_link_directories(target in_package)", checks=cc.link_directories)
+        self.assertEqual(["LINK_DIRECTORY"], result)
         result = mock_lint(env, pkg, "link_directories(/not/in/package)", checks=cc.link_directories)
         self.assertEqual(["EXTERNAL_LINK_DIRECTORY"], result)
+        result = mock_lint(env, pkg, "target_link_directories(target /not/in/package)", checks=cc.link_directories)
+        self.assertEqual(["EXTERNAL_LINK_DIRECTORY"], result)
 
     @posix_and_nt
     @patch("os.path.isfile", lambda x: x == os.path.normpath("/package-path/mock/FindLocal.cmake"))
@@ -245,6 +287,16 @@ class ChecksBuildTest(unittest.TestCase):
             """,
                            checks=cc.depends)
         self.assertEqual([], result)
+        result = mock_lint(env, pkg,
+                           """
+            project(mock)
+            find_package(catkin REQUIRED)
+            if(${CATKIN_ENABLE_TESTING})
+                find_package(other_catkin REQUIRED)
+            endif()
+            """,
+                           checks=cc.depends)
+        self.assertEqual(["AMBIGUOUS_CONDITION"], result)
         pkg = create_manifest("mock", build_depends=["other_catkin"], test_depends=["other_catkin"])
         result = mock_lint(env, pkg,
                            """
@@ -569,7 +621,7 @@ class ChecksBuildTest(unittest.TestCase):
         self.assertEqual(["REDUNDANT_LIB_PREFIX"], result)
 
     @posix_and_nt
-    @patch("os.path.isfile", lambda x: x in [os.path.normpath(f) for f in ["/package-path/mock/bin/script", "/package-path/mock/bin/script.in", "/package-path/mock/share/file", "/package-path/mock/src/source.cpp", "/some/external/script", "/some/external/file"]])
+    @patch("os.path.isfile", lambda x: x in [os.path.normpath(f) for f in ["/package-path/mock/bin/script", "/package-path/mock/bin/script.in", "/package-path/mock/share/file", "/package-path/mock/src/source.cpp", "/package-path/mock/include/source.h", "/some/external/script", "/some/external/file"]])
     @patch("os.path.isdir", lambda x: x in [os.path.normpath(d) for d in ["/package-path/mock/include", "/some/external/dir"]])
     def test_installs(self):
         """Test installation checks"""
@@ -740,6 +792,26 @@ class ChecksBuildTest(unittest.TestCase):
                            checks=cc.installs)
         self.assertEqual(["UNUSED_INCLUDE_PATH", "UNINSTALLED_INCLUDE_PATH"], result)
 
+        result = mock_lint(env, pkg,
+                           """
+            project(mock)
+            find_package(catkin REQUIRED)
+            catkin_package(INCLUDE_DIRS include)
+            install(FILES include/source.h DESTINATION ${CATKIN_PACKAGE_INCLUDE_DESTINATION})
+            """,
+                           checks=cc.installs)
+        self.assertEqual([], result)
+
+        result = mock_lint(env, pkg,
+                           """
+            project(mock)
+            find_package(catkin REQUIRED)
+            catkin_package(INCLUDE_DIRS include)
+            install(FILES include/source.h DESTINATION ${CATKIN_GLOBAL_INCLUDE_DESTINATION})
+            """,
+                           checks=cc.installs)
+        self.assertEqual(["HEADER_OUTSIDE_PACKAGE_INCLUDE_PATH"], result)
+
         result = mock_lint(env, pkg,
                            """
             project(mock)
@@ -817,10 +889,10 @@ class ChecksBuildTest(unittest.TestCase):
         open_func = "builtins.open" if sys.version_info[0] >= 3 else "__builtin__.open"
 
         # Work around a limitation of older Python mock_open() implementations
-        with patch(open_func, new_callable=mock_open, read_data="test\nthis\n"):
+        with patch(open_func, new_callable=mock_open, read_data="test\nthis\n"):  # pragma: no cover
             with open("anything", "r") as f:
                 if f.readline() != "test\n":
-                    return
+                    unittest.skip("known bug in old mock_open()")
 
         with patch(open_func, new_callable=mock_open, read_data="no python shebang\ncontent\n"):
             result = mock_lint(env, pkg,
@@ -1064,7 +1136,7 @@ class ChecksBuildTest(unittest.TestCase):
             catkin_package()
             """,
                            checks=cc.exports)
-        self.assertEqual(["SUGGEST_CATKIN_DEPEND"], result)
+        self.assertEqual(["MISSING_CATKIN_DEPEND"], result)
 
         pkg = create_manifest("mock")
         result = mock_lint(env, pkg,
