@@ -1,7 +1,37 @@
+# coding=utf-8
+#
+# catkin_lint
+# Copyright (c) 2013-2020 Fraunhofer FKIE
+#
+# Redistribution and use in source and binary forms, with or without
+# modification, are permitted provided that the following conditions
+# are met:
+#
+#  * Redistributions of source code must retain the above copyright
+#    notice, this list of conditions and the following disclaimer.
+#  * Redistributions in binary form must reproduce the above copyright
+#    notice, this list of conditions and the following disclaimer in the
+#    documentation and/or other materials provided with the distribution.
+#  * Neither the name of the Fraunhofer organization nor the names of its
+#    contributors may be used to endorse or promote products derived from
+#    this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+# IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
+# PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 import unittest
 import sys  # noqa
 import os
-from catkin_lint.linter import CMakeLinter, LintInfo, PathConstants
+from catkin_lint.linter import CMakeLinter, LintInfo, PathConstants, ERROR
 from .helper import create_env, create_manifest, mock_lint, patch, posix_and_nt
 import catkin_lint.checks.build as cc
 import catkin_lint.environment
@@ -190,7 +220,7 @@ class LinterTest(unittest.TestCase):
             if ("${varname}")
             endif()
             """, checks=cc.all)
-        self.assertEqual(["AMBIGUOUS_CONDITION"], result)
+        self.assertEqual([], result)
         result = mock_lint(env, pkg,
                            """
             project(mock)
@@ -230,6 +260,27 @@ class LinterTest(unittest.TestCase):
             """, checks=cc.all, package_path="/package-path/other")
         self.assertEqual(["PACKAGE_PATH_NAME"], result)
 
+    def test_report_path(self):
+        """Test path normalization for diagnostic messages"""
+        env = create_env()
+        info = LintInfo(env)
+        self.assertEqual(info.report_path("test"), "test")
+        self.assertEqual(info.report_path("foo/../bar"), "bar")
+        self.assertEqual(info.report_path("/absolute//stuff"), "/absolute/stuff")
+        self.assertEqual(info.report_path("%s/test" % PathConstants.PACKAGE_SOURCE), "test")
+        self.assertEqual(info.report_path("%s/test" % PathConstants.PACKAGE_BINARY), "${CMAKE_BINARY_DIR}/test")
+        self.assertEqual(info.report_path("%s/%s/test" % (PathConstants.PACKAGE_SOURCE, PathConstants.PACKAGE_BINARY)), "${CMAKE_SOURCE_DIR}/${CMAKE_BINARY_DIR}/test")
+        self.assertEqual(info.report_path("%s/%s/test" % (PathConstants.PACKAGE_BINARY, PathConstants.PACKAGE_SOURCE)), "${CMAKE_BINARY_DIR}/${CMAKE_SOURCE_DIR}/test")
+        self.assertEqual(info.report_path("%s/test" % PathConstants.CATKIN_DEVEL), "${CATKIN_DEVEL_PREFIX}/test")
+        self.assertEqual(info.report_path("%s/test" % PathConstants.CATKIN_INSTALL), "${CATKIN_INSTALL_PREFIX}/test")
+        self.assertEqual(info.report_path(info.find_package_path("test", "include")), "${test_INCLUDE_DIRS}")
+        self.assertEqual(info.report_path(info.find_package_path("test", "lib/library.so")), "${test_LIBRARIES}")
+        self.assertEqual(info.report_path("%s//test" % PathConstants.PACKAGE_BINARY), "${CMAKE_BINARY_DIR}/test")
+        self.assertEqual(info.report_path("%s/bar/../foo" % PathConstants.PACKAGE_BINARY), "${CMAKE_BINARY_DIR}/foo")
+        self.assertEqual(info.report_path("%s/../test" % PathConstants.PACKAGE_BINARY), "${CMAKE_BINARY_DIR}/../test")
+        self.assertEqual(info.report_path("%s/../test" % PathConstants.PACKAGE_BINARY), "${CMAKE_BINARY_DIR}/../test")
+        self.assertEqual(info.report_path("//%s%s/bar/../foo" % (PathConstants.CATKIN_DEVEL, PathConstants.PACKAGE_BINARY)), "/${CATKIN_DEVEL_PREFIX}${CMAKE_BINARY_DIR}/foo")
+
     def test_list(self):
         """Test CMake list handling"""
         env = create_env()
@@ -326,7 +377,7 @@ class LinterTest(unittest.TestCase):
     @patch("os.path.realpath", lambda x: x)
     def test_environment(self):
         """Test catkin environment"""
-        env = catkin_lint.environment.CatkinEnvironment(use_rosdep=False)
+        env = catkin_lint.environment.CatkinEnvironment(use_rosdep=False, quiet=True)
         mock_packages = {}
         mock_packages[os.path.normpath("/mock_catkin")] = create_manifest("mock_catkin")
         mock_packages[os.path.normpath("/mock_other")] = create_manifest("mock_other")
@@ -334,23 +385,34 @@ class LinterTest(unittest.TestCase):
         with patch("catkin_lint.environment.find_packages", lambda x, use_cache: mock_packages):
             result = env.add_path(os.path.normpath("/"))
             self.assertEqual(1, len(result))
+            self.assertEqual(env.get_package_type("mock_catkin"), catkin_lint.environment.PackageType.CATKIN)
             self.assertTrue(env.is_catkin_pkg("mock_catkin"))
+            self.assertTrue(env.is_known_pkg("mock_catkin"))
+            self.assertEqual(env.get_package_type("mock_other"), catkin_lint.environment.PackageType.OTHER)
             self.assertFalse(env.is_catkin_pkg("mock_other"))
+            self.assertTrue(env.is_known_pkg("mock_other"))
             result = env.add_path(os.path.normpath("/"))
             self.assertEqual(1, len(result))
-            self.assertTrue(env.is_catkin_pkg("mock_catkin"))
-            self.assertFalse(env.is_catkin_pkg("mock_other"))
+            self.assertEqual(env.get_package_type("mock_catkin"), catkin_lint.environment.PackageType.CATKIN)
+            self.assertEqual(env.get_package_type("mock_other"), catkin_lint.environment.PackageType.OTHER)
             result = env.add_path(os.path.normpath("/missing"))
             self.assertEqual([], result)
-            self.assertFalse(env.is_catkin_pkg("invalid"))
+            env.knows_everything = True
+            self.assertTrue(env.ok)
+            self.assertEqual(env.get_package_type("invalid"), catkin_lint.environment.PackageType.UNKNOWN)
+            self.assertFalse(env.is_known_pkg("invalid"))
+            env.knows_everything = False
+            self.assertFalse(env.ok)
+            self.assertEqual(env.get_package_type("invalid"), catkin_lint.environment.PackageType.INDETERMINATE)
+            self.assertFalse(env.is_known_pkg("invalid"))
 
-        def raiseError():
+        def raiseError():  # pragma: no cover
             raise RuntimeError()
         with open(os.devnull, "w") as devnull:
             with patch("catkin_lint.environment.get_rosdep", raiseError):
                 with patch("sys.stderr", devnull):
                     env = catkin_lint.environment.CatkinEnvironment()
-                    self.assertFalse(env.ok)
+                    self.assertFalse(env.knows_everything)
         self.assertFalse(catkin_lint.environment.is_catkin_package(None))
 
     @posix_and_nt
@@ -374,6 +436,13 @@ class LinterTest(unittest.TestCase):
                            )
         self.assertEqual([], result)
 
+        result = mock_lint(env, pkg,
+                           {
+                               "/package-path/mock/CMakeLists.txt": "project(mock) add_subdirectory(src)",
+                           }, checks=cc.all
+                           )
+        self.assertEqual(["OS_ERROR"], result)
+
         result = mock_lint(env, pkg,
                            {
                                "/package-path/mock/CMakeLists.txt": "project(mock) add_subdirectory(src)",
@@ -465,3 +534,36 @@ class LinterTest(unittest.TestCase):
                         }, checks=None, return_var=True
                         )
         self.assertEqual("subdir", var["foo"])
+
+    def test_register_message(self):
+        """Test custom messages"""
+
+        def custom_invalid_id(linter):
+            linter.register_message("invalid", "invalid message")
+
+        def custom_invalid_text(linter):
+            linter.register_message("X_INVALID", "message text has \n newline in it")
+
+        def custom_duplicate_id(linter):
+            linter.register_message("X_DUPLICATE", "message")
+            linter.register_message("X_DUPLICATE", "different message")
+
+        def custom_empty_text(linter):
+            linter.register_message("X_TEST", "")
+
+        def custom_ok(linter):
+            def on_final(info):
+                info.report(ERROR, "X_TEST")
+            linter.register_message("X_TEST", "test message")
+            linter.add_final_hook(on_final)
+
+        env = create_env()
+        pkg = create_manifest("mock")
+        self.assertRaises(ValueError, mock_lint, env, pkg, "", checks=custom_invalid_id)
+        self.assertRaises(ValueError, mock_lint, env, pkg, "", checks=custom_invalid_text)
+        self.assertRaises(ValueError, mock_lint, env, pkg, "", checks=custom_empty_text)
+        self.assertRaises(ValueError, mock_lint, env, pkg, "", checks=custom_duplicate_id)
+        result = mock_lint(env, pkg, "", checks=custom_ok)
+        self.assertEqual(["X_TEST"], result)
+        result = mock_lint(env, pkg, "", checks=custom_ok)
+        self.assertEqual(["X_TEST"], result)
